/*  Copyright © 2016 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 29/08/16.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __mc_engine_opts_h__
#define __mc_engine_opts_h__ 1


/*   std includes   */
#include <map>
#include <vector>
#include <climits>
#include <iostream>

/*   project headers   */
#include <bits/duct-tags.h>



namespace duct {
 
  enum class evolution_type : int {
    typeI = 1,
    typeII = 2
  };
 
  
  /*   Simple structure to provide input parameters for the shower engine.   */
  struct shower_engine_opts
  {
    /*   for test purpuses we might want to use kind of insane options    */
    bool is_insane = false;
    
    /*   cutoff kT scale   */
    double kT_cutoff = 1.0_GeV;
    
    /*   virtuality cutoff scale    */
    double virtuality_cutoff = 1.0_GeV;

 
    /*    Type of the evolution   */
    evolution_type type_of_color_evolution = evolution_type::typeI;
    
    /*   max number of the basis state to consider in every steps   */
    unsigned int max_number_of_color_basis_states = 2u;
    
    
    
    /*   parameters to control the color approximations    */
    struct color_opts
    {
      color_opts(unsigned suppr = 0u, bool ipi = false, unsigned re = 0u)
      {
        max_color_suppression = suppr;
        soft_corrections = re;
        ipi_effects = ipi;

        type_of_evolution = evolution_type::typeI;
        max_number_of_basis_states = 2u;
      }
      
     
      /*   max allowed color suppression   */
      unsigned int max_color_suppression;

      /*    powers of the soft corrections    */
      unsigned int soft_corrections;
    
      /*   consider i*pi effect   */
      bool ipi_effects;

      /*    Type of the evolution   */
      evolution_type type_of_evolution;
      
      /*   max number of the basis state to consider in every steps   */
      unsigned int max_number_of_basis_states;
    };
    
  
    /*   color option for consecutive evolution intervals   */
    std::map<double, color_opts, std::greater<double>> color;

  
    struct threshold_opts
    {
      /*   enamble threshold contributions   */
      bool enabled = true;
      
      /*  controling Delta V expansion in the threshold operator    */
      unsigned int order_of_soft_corrections = 0u;
    } threshold;
  
    
    /*   clean up the shower options   */
    shower_engine_opts& sanitize();
  };
  
  
  /*   fancy print   */
  std::ostream& operator<<(std::ostream&, const shower_engine_opts&);

}

#endif
