/*  Copyright (C) 2015 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/8/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __duct_module_h__
#define __duct_module_h__ 1

/*   std includes   */
#include <functional>
#include <initializer_list>
#include <string>
#include <vector>
#include <list>


/*   project headers   */



namespace duct {
  
  enum class module_type {
    none = -1,
    init = 0,
    process = 1,
    finish = 2,
    other
  };
  
  
  /*   Simple structure to export module functions   */
  template<class> struct module_entry;
  
  
  template<typename Ret, typename ...Args>
  struct module_entry<Ret(Args...)>
  {
    /*   constructors   */
    template<typename _StringT, typename _FuncT>
    module_entry(module_type t, _StringT&& n, _FuncT&& f)
    : type(t), name(std::forward<_StringT>(n)) , func(std::forward<_FuncT>(f)) {}
 
    
    template<typename _StringT, typename _FuncT>
    module_entry(_StringT&& n, _FuncT&& f)
    : type(module_type::process), name(std::forward<_StringT>(n)) , func(std::forward<_FuncT>(f)) {}
    
   
    /*   data members  */
    module_type type;
    std::string name;
    std::function<Ret(Args...)> func;
  };
  
  
  /*   Global object to store the exported modules   */
  template<class Func>
  std::list<std::vector<module_entry<Func>>> _G_exported_modules{};

  
  /*   This class helps to export a list of modules. It has to be called manually
   *   or put into .init or .ctor or something similar section of the module
   *   shared object.
   */
  template<class> class export_module;
  
  
  template<typename Ret, typename ...Args>
  class export_module<Ret(Args...)>
  {
    /*    private types   */
    using _Iterator = typename std::list<std::vector<module_entry<Ret(Args...)>>>::iterator;

  public:
    /*   no default constroctor or copy or move */
    export_module() = delete;
    export_module(export_module&&) = delete;
    export_module(const export_module&) = delete;

    export_module& operator=(export_module&&) = delete;
    export_module& operator=(const export_module&) = delete;
    
    /*    construcor   */
    export_module(std::initializer_list<module_entry<Ret(Args...)>> tbl) {
      _M_current = _G_exported_modules<Ret(Args...)>.emplace(_G_exported_modules<Ret(Args...)>.end(), tbl);
    }
    
    /*    destructor   */
    ~export_module() {
      _G_exported_modules<Ret(Args...)>.erase(_M_current);
    }
        
  private:
    /*   the current module that was inserted by this class   */
    _Iterator _M_current;
  };
  
  
}


#endif
