/*  Copyright (C) 2014 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/28/14.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __mc_hard_params_h__
#define __mc_hard_params_h__ 1



/*   std includes   */


/*   project headers   */
#include <bits/duct-tags.h>
#include <cxx/stackvector.h>


namespace duct {
 
  /*   Represents a set of hard parameters.
   *   Here we have the simple variables for initial and final state.
   */
  template<IniFinTag> struct hard_params;
  
  
  template<>
  struct hard_params<fin>
  {
    /*   index of the emitter parton   */
    int emitter;
    
    /*   actual hard parameters   */
    int fl;
    double mT, ymax, al;
    energy_sqr<double> pQ, Q2;

    /*   veto parameters   */
    double mTmax, mYmax;

    /*   spectator variables   */
    struct spectator_vars
    {
      int spectator;
      double xi, cfac;
    };
    
    /*   considering all the possible spectators    */
    stackvector<spectator_vars, 5u> spectators;
  };
  

  template<>
  struct hard_params<ini>
  {
    /*   index of the emitter parton   */
    int emitter;
    
    /*   actual hard parameters   */
    int fl;
    double eta, mT;
    energy_sqr<double> Q2;

    /*   veto parameters   */
    double mTmax, mYmax;
   
    
    /*   spectator variables   */
    struct spectator_vars
    {
      int spectator;
      double xi, cfac;
    };
    
    /*   considering all the possible spectators    */
    stackvector<spectator_vars, 5u> spectators;
  };
  
 
}    //    namespace duct


#endif

