/*  Copyright (C) 2014 Zoltán Nagy
 *  Created by Zoltán Nagy on 11/23/14.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __mc_ordering_lmd_h__
#define __mc_ordering_lmd_h__ 1


/*   std headers   */
#include <cmath>
#include <utility>

/*   project headers   */
#include <bits/mc-hard-params.h>



namespace duct {
  
  template<>
  struct ordering_traits<fin, ordering::lambda>
  {
    /*   ordering specific properties in the main shower    */
    struct main
    {
      /*  calculates the limits on the variable v    */
      static auto v2limits(const hard_params<fin>& pars) {
        return std::pair{4.0*pars.mT*pars.Q2, std::min(pars.ymax, pars.mYmax)*pars.Q2};
      }
   
      /*    calculates the z limit   */
      static auto zlimits(const hard_params<fin>& pars, energy_sqr<double> v2)
      {
        double y = v2/pars.Q2;
        double zp = (pars.mT >= 0.25*y ? 0.4999999 : 0.5+std::sqrt(0.25-pars.mT/y));
        return std::pair{1.0-zp, zp};
      }
      
      /*    mapping the independent variables to y   */
      static auto mapping_to_y(const hard_params<fin>& pars, energy_sqr<double> v2, double) {
        return std::pair{v2/pars.Q2, 1.0};
      }
    };
    
    /*    we might need other components    */
  };
  
  
  
  
  template<>
  struct ordering_traits<ini, ordering::lambda>
  {
    /*   ordering specific properties in the main shower    */
    struct main
    {
      /*  calculates the limits on the variable v    */
      static auto v2limits(const hard_params<ini>& pars)
      {
        const double &eta = pars.eta;
        return std::pair{pars.mT*pars.Q2/(1.0-eta), std::min(1.0/eta-1.0, pars.mYmax)*pars.Q2};
      }
   
      /*    calculates the z limit   */
      static auto zlimits(const hard_params<ini>& pars, energy_sqr<double> v2)
      {
        const double y = v2/pars.Q2;
        return std::pair{pars.eta, std::min(1.0/(1.0+y), 1.0-pars.mT/y)};
      }
      
      /*    mapping the independent variables to y   */
      static auto mapping_to_y(const hard_params<ini>& pars, energy_sqr<double> v2, double) {
        return std::pair{v2/pars.Q2, 1.0};
      }
    };
    
    /*    we might need other components    */
  };
  
}    //   namespace duct 

#endif
