/*  Copyright © 2021 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 17.01.21 at 15:32.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __cxx_bounded_array_h__
#define __cxx_bounded_array_h__ 1


/*   std includes   */
#include <array>


/*   project headers   */


namespace duct {
  
  template<typename _Tp, std::ptrdiff_t _Low,  std::ptrdiff_t _High>
  class bounded_array : public std::array<_Tp, _High-_Low+1>
  {
    using _Base = std::array<_Tp, _High-_Low+1>;
    
  public:
    using difference_type = std::ptrdiff_t;
    
    /*    Element access    */
    constexpr typename _Base::reference operator[](difference_type k) {
      return _Base::operator[](k-_Low);
    }

    constexpr typename _Base::const_reference operator[](difference_type k) const {
      return _Base::operator[](k-_Low);
    }

    /*    Element access with check    */
    constexpr typename _Base::reference at(difference_type k) {
      return _Base::at(k-_Low);
    }

    constexpr typename _Base::const_reference at(difference_type k) const {
      return _Base::at(k-_Low);
    }
  };
  
  
  
}  //  namespace duct

#endif

