/*  resum-exp.cc
 *  Created by Zoltan Nagy on 25.01.20.
 *  Copyright © 2020 Zoltán Nagy. All rights reserved.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


/*   std includes    */


/*   project headers    */
#include "bits/rs-resum-exp.h"




namespace duct {
  namespace resum {
    
    namespace __details {
      
      /*    some simple functions to test color connections    */
      template<ProcessTag _Proc>
      inline int tauL(int i, int l, const color_state<_Proc>& p) {
        return (l == i ? 1 : 0) - (p[l].colorL.prev == i ? 1 : 0);
      }
      
      
      template<ProcessTag _Proc>
      inline int tauR(int i, int l, const color_state<_Proc>& p) {
        return (l == i ? 1 : 0) - (p[l].colorR.prev == i ? 1 : 0);
      }
    }
    

    
    void resummation_exponent<epa>::_M_color_mapping(int l, int k, shower_color_state<epa>& c)
    {
      auto ic = c.begin(), ilast = --c.end();
      
      do
      {
        /*   clear the list fo vertices    */
        _TMP_color_vertices.clear();
        
        
        /*   finding the relevant color vertices    */
        int v1 = 0;
        const color_state<epa>& ci = *ic;
        int nparton = ci.upper();
        
        for(int r = 1; r <= nparton; ++r)
          if(ci.is_antitriplet(r) == false)
          {
            for(int s = 1; s <= nparton; ++s)
              if(ci.is_antitriplet(s) == false)
              {
                int vL = __details::tauL(r,l, ci)*__details::tauL(s,k, ci);
                int vR = __details::tauR(r,l, ci)*__details::tauR(s,k, ci);
                
                v1 += vL + vR;
                
                /*   nothing to do if the gluon insertion leads to tadpoles   */
                if(ci[r].colorL.next == s || ci[s].colorL.next == s) vL = 0;
                if(ci[r].colorR.next == s || ci[s].colorR.next == r) vR = 0;
                
                /*    sorting the vertices   */
                if(vL) _TMP_color_vertices.push_back({'G','L', r, s, 0.5*vL, 0, 0});
                if(vR) _TMP_color_vertices.push_back({'G','R', r, s, 0.5*vR, 0, 0});
              }
          }
        
        
        /*    do the transformation   */
        if(_TMP_color_vertices.size())
          c.transform(ic, _TMP_color_vertices.begin(), _TMP_color_vertices.end());
        
        /*    the subleading part    */
        ic->weight *= -v1*0.5/qcd::Nc;
        
      } while(ic++ != ilast);
    }
    

   }  //  namespace resum
}  //  namespace duct

